<concordance-timeline>
   <div if={isLoading}
         class="center-align">
      <preloader-spinner center={true}/>
   </div>
   <div if={error}>
      <h6>{_("loadingTimelineFaild")}</h6>
      {error}
   </div>
   <div style="min-height: 400px;">
      <external-text text="timelines"/>
      <div class="flex pl-10 mt-4" style="gap: 30px; flex-wrap: wrap; align-items: baseline;">
         <ui-checkbox label-id="showAbsFreq"
               inline={true}
               checked={data.t_absfrq}
               on-change={onAbsfrqChange}></ui-checkbox>
         <ui-select if={corpus.diachronic.length > 1}
               options={corpus.diachronic}
               classes="mr-10"
               label={_("period")}
               inline={true}
               riot-value={data.t_attr}
               on-change={onAttrChange}></ui-select>
      </div>
      <div ref="chart"></div>
   </div>

   <script>
      const {Connection} = require("core/Connection.js")
      const {AppStore} = require("core/AppStore.js")
      this.mixin("feature-child")

      this.isLoading = true
      this.items = null

      this.on("before-mount", () => {
         this.attr = this.corpus.diachronic[0].value
         AppStore.loadGoogleCharts()
         this.loadData()
         AppStore.on("GOOGLE_CHARTS_LOADED", this.redrawChart.bind(this))
      })
      this.on("update", () => {
         if(!window.objectEquals(this.lastConcordanceQuery, this.store.getConcordanceQuery())){
            this.loadData()
         }
      })

      this.on("before-unmount", () => {
         AppStore.off("GOOGLE_CHARTS_LOADED", this.redrawChart.bind(this))
      })

      onAttrChange(attr){
         this.data.t_attr = attr
         this.loadData()
         this.store.updateUrl()
         this.store.saveUserOptions(["t_attr"])
      }

      onAbsfrqChange(absfrq){
         this.data.t_absfrq = absfrq
         this.redrawChart()
         this.store.updateUrl()
         this.store.saveUserOptions(["t_absfrq"])
      }

      loadData(){
         let query = this.store.getConcordanceQuery()
         this.lastConcordanceQuery = query
         this.isLoading = true
         this.update()
         Connection.get({
            url: window.config.URL_BONITO + "freqml",
            data: {
               corpname: this.store.corpus.corpname,
               fmaxitems: 5000,
               fpage: 1,
               freq_sort: "freq",
               group: 0,
               showpoc: 0,
               showreltt: 0,
               showrel: 0,
               freqlevel: 1,
               ml1attr: this.data.t_attr,
               ml1ctx: "0~0>0",
               format: "json",
               concordance_query: query
            },
            done: this.onDataLoaded.bind(this),
            fail: this.onDataFailed.bind(this),
            always: () => {
               this.isLoading = false
               this.update()
            }
         })
      }

      onDataLoaded(payload){
         if(payload.error){
            this.error = payload.error
         } else{
            this.items = payload.Blocks[0].Items.filter(item => item.Word[0].n != "===NONE===")
                  .map(item => {
                     return {
                        w: item.Word[0].n,
                        frq: item.frq,
                        reltt: item.reltt
                     }
                  })
            this.items.sort((a, b) => {
               return a.w < b.w ? -1 : 1
            })

            this.update()
            this.redrawChart()
         }
      }

      onDataFailed(payload){
         this.error = payload.error
      }

      redrawChart(){
         if(this.items && AppStore.isGoogleChartsLoaded){
            var dataTable = new google.visualization.DataTable()
            dataTable.addColumn("string", _("period"))
            dataTable.addColumn("number", _("relativeFrequency"))
            this.data.t_absfrq && dataTable.addColumn("number", _("absoluteFrequency"))
            dataTable.addRows(this.getChartData())

            let options = {
               vAxis: {
                  baseline: 0,
                  textStyle: {
                     fontSize: 12
                  }
               },
               hAxis: {
                  textStyle: {
                     fontSize: 12
                  }
               },
               vAxes: [{
                  minValue: 0
               }],
               backgroundColor: {
                  fill:"transparent"
               },
               legend: {
                  position: "bottom"
               },
               focusTarget: "category",
               height: 400,
               chartArea: {
                  left: 70,
                  top: 20,
                  right: this.data.t_absfrq ? 80 : 20,
                  "width": "90%",
                  "height": "75%"
               },
               bar: {
                  groupWidth: "50%"
               }
            }
            if(this.data.t_absfrq){
               options.series = {
                    0: {
                        targetAxisIndex: 0,
                        type: "line"
                    },
                    1: {
                        targetAxisIndex: 1,
                        type: "bars",
                        color: "cccccc"
                    }
                }
                options.vAxes[0].title = _("relativeFrequency")
                options.vAxes.push({
                    title: _("absoluteFrequency"),
                    minValue: 0/*,
                    viewWindow:{
                        max: this.getAxisMaxValue(data, 1)
                    }*/
                })
            }
            let chart = new google.visualization.ComboChart(this.refs.chart)
            chart.draw(dataTable, options)
            google.visualization.events.addListener(chart, "select", this.onChartSelect)
            this.chart = chart
         }
      }

      onChartSelect(evt){
         let attrValue = this.items[this.chart.getSelection()[0].row].w
         let tmp = this.attr.split(".")
         Dispatcher.trigger("FEATURE_TOOLBAR_HIDE_OPTIONS")

         this.store.filter({
            pnfilter: "p",
            inclkwic: true,
            filfpos: "0",
            filtpos: "0",
            cql: `[] within <${tmp[0]} ${tmp[1]}="${attrValue}" />`,
            queryselector: "cqlrow"
         })
      }

      getChartData(word){
         return this.items.map(item => {
            if(this.data.t_absfrq){
               return [item.w, item.reltt, item.frq]
            } else {
               return [item.w, item.reltt]
            }
         })
      }

   </script>
</concordance-timeline>
